#include <chplot.h>
#include <assert.h>
#include "boxplot.h"

/* source code for boxplot */

/** These are internal implementation functions */
CBoxplot::CBoxplot()
{
    numdata = 0;
    dptr_data = NULL;
    median = 0;
    upperQ=0;
    lowerQ = 0;
    fs = 0;
    end_up = 0;
    end_low = 0;
    numOL_low = 0;
    numOL_up = 0;
    Outlier_up=new double[100];
    Outlier_low=new double[100];
}
CBoxplot::~CBoxplot()
{
	delete[] dptr_data;
        delete[] Outlier_up;
        delete[] Outlier_low;
}
void CBoxplot::SetData(double dPtr[],int numD){
	//Initialization of the data
	dptr_data=NULL;
	dptr_data=dPtr;
	numdata=numD;
	if(numD>2){
		estimateStat();
	}
}



void CBoxplot::estimateStat(){
 	if (dptr_data==NULL) return;
	//double* dptr1; 
	double* dptr_Low;
	double* dptr_Up;
	double dt1,dt2;
        int i;
	dptr_Low=new double[numdata];
	dptr_Up=new double[numdata];
	double* sdata_t1=new double [numdata];
	for (i=0;i<numdata;i++) sdata_t1[i]=dptr_data[i];
	sorting(sdata_t1,numdata);
	median=est_median(sdata_t1,numdata);
	int nUp = 0,nLow =0;
	if(Odd(numdata)){
		for(i=0;i<(numdata+1)/2;i++)
		{	
			dt1=sdata_t1[i];++nLow;
			dptr_Low[nLow-1]=dt1;
			dt2=sdata_t1[(numdata+1)/2+i-1];++nUp;
			dptr_Up[nUp-1]=dt2;
		}
	}
	else
	{	
		for(i=0;i<(numdata)/2;i++)
		{
			dt1=sdata_t1[i];++nLow;
			dptr_Low[nLow-1]=dt1;
			dt2=sdata_t1[(numdata+1)/2+i];++nUp;
			dptr_Up[nUp-1]=dt2;
		}
	}
		lowerQ=est_median(dptr_Low,nLow);	
		upperQ=est_median(dptr_Up,nUp);	
		fs=upperQ-lowerQ;

		end_up=UpLow_end(dptr_data,numdata,0);
		end_low=UpLow_end(dptr_data,numdata, 1);
}


bool CBoxplot::Odd(int numD){
	double x=(double)numD/2.0;
	int xint=numD/2;
	if (x==(double)xint)
		return false;
	else
		return true;
}

double CBoxplot::est_median(double dptr[],int numD){
	double med;
	if(Odd(numD)){
		med=dptr[(numD+1)/2-1];	
	}
	else 
	{
		med=(dptr[numD/2-1]+dptr[numD/2+1-1])/2.0;
	}
	return med;
}

// UpLow_end
// Up and Low end mark for Boxplot
// if b1 is 0 then it is for up if b1=1, for low
double CBoxplot ::UpLow_end(double dP[],int numD,int b1)
{
	
	double maxDouble =0,minDouble=0;
	double* data_temp=new double [numdata];
	double* sdata_t1=new double [numdata];
        int i;

	int num_t1=numdata;
	if (b1==0)//for upper end
	{	
		for (i=0;i<numdata;i++) sdata_t1[i]=dptr_data[i];
		sorting(sdata_t1,numdata);
		maxDouble=sdata_t1[numdata-1];
		numOL_up=0;
		while((1.5*fs+upperQ)<maxDouble)
		{	
			Outlier_up[numOL_up]=maxDouble;
			++numOL_up;
			--num_t1;

			maxDouble=sdata_t1[num_t1-1];
			if(num_t1==0)
			{	maxDouble=upperQ;
				break; 		     
			}
			if(numOL_up>100){
				break;
			}
		}

		return maxDouble;
	}
	else if(b1==1)// for lower end
	{	
		for (i=0;i<numdata;i++)
		{
			data_temp[i]=-1.0*dP[i];
		}
		sorting(data_temp,numdata);
		double maxDouble=0;
		maxDouble=data_temp[numdata-1];
		numOL_low=0;
		num_t1=numdata;
		while((lowerQ-1.5*fs)>(-maxDouble))
		{	
			Outlier_low[numOL_low]=-maxDouble;
			++numOL_low;
			--num_t1;
			maxDouble=data_temp[num_t1-1];
			if(num_t1==0)
			{	maxDouble=lowerQ;
				break; 		     
			}
			if(numOL_low>100){
			break;
			}
		}

		return -maxDouble;
	}
	return 0;

}


void CBoxplot::sorting(double dP[],int numD)
{
        int i;
	double temp =0;
	int pass;
	for (pass=0;pass<numD;pass++)
	{
		for(i=0;i<numD-1;i++)
		{
			if(dP[i]>dP[i+1]){
				temp=dP[i];
				dP[i]=dP[i+1];
				dP[i+1]=temp;
			}
		}
	}
}

double CBoxplot::maximum(double dP[],int numD)
{	double max_d=(-100000000000000000);
        int i;
	for (i=0;i<numD;i++){
		if (dP[i]>=max_d)
			max_d=dP[i];
	}
	return max_d;
}

double CBoxplot::minimum(double dP[],int numD)
{	double min_d=(100000000000000000);
        int i;
	for (i=0;i<numD;i++){
		if (dP[i]<=min_d)
			min_d=dP[i];
	}
	return min_d;
}


/* this is an external function */
// Boxplot 
// Example
//   boxplot(CPlot *plot, double_array_pointer,number_of_data,number_of_seperation);
//       plot.plotting();

// Reference : Probability and Statistics for Engineering and the Sciences, 4e
//              Jay L. Devore,1995, pp 22, 34
// Code :
// Written by Taesam Lee, Sep 2006, 
// Colorado State University, Department of Civil Engineering, Ph.D.

// boxplot(CPlot *plot,double_array_pointer,number_of_data,number_of_seperation);
// number of data=(number of seperation)*(number of data for each data set)

int boxplot(CPlot *plot, double x[],int n,int numSep){            
                // number of Separation for the data compromise the number of data

                // * n/numSep should be integer
                int nInt=n/numSep;
                int i, is, i2;
                double nDouble=(double)n/(double)numSep;
                int num = 0; // number of curves
                assert(nInt==nDouble);

                 int numData=n/numSep;
                 double* xTemp=new double[numData];
                 CBoxplot bp1;
                 for (is=1;is<=numSep;is++){
                         for(i2=0;i2<(numData);i2++)
                         {xTemp[i2]=x[numData*(is-1)+i2];}
                   bp1.SetData(xTemp,numData);
                   
                   plot->rectangle(0.75+(is-1),bp1.lowerQ,0.5,bp1.fs);
                   plot->plotType(PLOT_PLOTTYPE_LINES,num,3,1);
                   num++;
                   //straight line from box tplot to 1.5fs
                   plot->line(is,bp1.lowerQ,0,is,bp1.end_low,0);
                   plot->plotType(PLOT_PLOTTYPE_LINES,num,3,1);
                   num++;
                   //straight line from boxt plot to 1.5fs
                   plot->line(is,bp1.upperQ,0,is,bp1.end_up,0);
                   plot->plotType(PLOT_PLOTTYPE_LINES,num,3,1);
                   num++;
                   plot->line(is-0.25,bp1.median,0,is+0.25,bp1.median,0);
                   plot->plotType(PLOT_PLOTTYPE_LINES,num,3,1);
                   num++;

                   if(bp1.numOL_low>0)
                   {
                           double* xlowOL=new double[bp1.numOL_low];
                           for(i=0;i<bp1.numOL_low;i++){
                                   xlowOL[i]=is;
                           }
                           plot->data2DCurve(xlowOL,bp1.Outlier_low,bp1.numOL_low);
                           plot->plotType(PLOT_PLOTTYPE_POINTS,num,3,1);
                           num++;
                          delete[] xlowOL;
                   }

                   if(bp1.numOL_up>0)
                   {
                           double* xupOL=new double[bp1.numOL_up];
                           for(i=0;i<bp1.numOL_up;i++){
                                   xupOL[i]=is;
                           }
                           plot->data2DCurve(xupOL,bp1.Outlier_up,bp1.numOL_up);
                           plot->plotType(PLOT_PLOTTYPE_POINTS,num,3,1);
                           num++;
                           delete[] xupOL;
                   }
                 }// for numSep
                 plot->axisRange(PLOT_AXIS_X,0,(double)numSep+1.0,1);
                 //delete[] xTemp;
                 //bp1.Outlier_low=NULL;
                 //bp1.Outlier_up=NULL;
                 return 0;
}
